

import SwiftUI
//要求Item符合Identifiable,ItemView则符合View
struct AspectVGrid<Item,ItemView>: View where ItemView:View,Item:Identifiable {
    var items:[Item]//Item数组（game.cards）
    var aspctRatio:CGFloat //.aspctRatio的参数(2/3)
    var content:(Item) -> ItemView //内容闭包处理
    //手工初始化以增加@ViewBuilder
    init(items:[Item],aspctRatio:CGFloat,@ViewBuilder content:@escaping (Item) -> ItemView) {
        self.items = items
        self.aspctRatio = aspctRatio
        self.content = content
    }
    var body: some View {
        GeometryReader{ geometry in //获取当前视图的尺寸
            VStack{//为了使下面的子视图对齐到可用空间的顶部
                //让其在GeometryReader中的内容大小上是灵活的
                Spacer()//向下挤，让LazyVGrid居中
                //width通过widthThatFits计算后返回一个合适的大小以匹配屏幕的尺寸
                let width = widthThatFits(itemCount: items.count, in: geometry.size, itemAspectRatio: aspctRatio)
                LazyVGrid(columns: [adaptiveeGridItem(width: width)]){
                    ForEach(items){ item in
                        //处理闭包要显示的内容
                        content(item).aspectRatio(aspctRatio,contentMode: .fill)
                    }
                }
                Spacer()//向上挤，让LazyVGrid居中
            }
        }        
    }
    //通过重新定义GridItem来修改spacing的默认值(其它视图也可以这样修改)
    private func adaptiveeGridItem(width:CGFloat) -> GridItem{
        var gridItem = GridItem(.adaptive(minimum: width))
        gridItem.spacing = 0 //设置Grid的边距(本函数的主要功能)
        return gridItem
    }
    //卡片宽度计算(符合屏幕宽度)
    private func widthThatFits(itemCount:Int,in size:CGSize,itemAspectRatio:CGFloat) -> CGFloat{
        var columnCount = 1 //一行显示几列？
        var rowCount = itemCount //格子数(一格一个卡片）
        //通过循环对比算出来显示让一行显示几列需要的合适宽度
        repeat{
            let itemWidth = size.width / CGFloat(columnCount)//根据上级容器的宽得出单个卡片的宽
            let itemHeight = itemWidth / itemAspectRatio//根据显示比较得出卡片的高
            if CGFloat(rowCount) * itemHeight < size.height{
                break //格子数*卡片高的情况下小于了容器高度
            }
            columnCount += 1 //满足条件 +1
            //重新计算格子数量
            rowCount = (itemCount + (columnCount - 1)) / columnCount
        } while columnCount < itemCount //满足列 > 卡片数则继续循环
            //防止列数大于实际的卡片数量
            if columnCount > itemCount{ //如果 列 > 卡片数
                columnCount = itemCount //列数与卡片一样多(单行显示)
            }
        //返回单个卡片需要的宽度 (整体宽度 / 计算后的列数)
        return floor(size.width / CGFloat(columnCount))//核心就是要算出来一行显示几列的宽度
    }
}

