//
//  Cardify.swift
//  Memorize
//
//  Created by kwok on 2021/8/24.
//

import SwiftUI
//AnimatableModifier包含了Animatable与ViewModifier
struct Cardify:AnimatableModifier {
    init(isFaceUp:Bool) {
        rotation = isFaceUp ? 0 : 180
    }
    var rotation: Double //旋转值(我们需要通过animatableData让此值动画化)
    //使用animatableData为了符合Animatable协议
    var animatableData: Double{ //Double符合VectorArithmetic协议
        get{ rotation }//需要动画化的值
        set{
            rotation = newValue
           // print(rotation)
        }//animatableData 会记录动画运行时数据不断变化的值
    }
    func body(content: Content) -> some View {
        ZStack{
            let shape = RoundedRectangle(cornerRadius: DrawingConstants.cornerRadius)
            if rotation < 90{ //通过旋转值判断翻转
                shape.fill().foregroundColor(.white)
                shape.strokeBorder(lineWidth: DrawingConstants.lineWidth)
            }else{
                shape.fill()
            }
            content.opacity(rotation < 90 ? 1 : 0)//当旋转值小于90时才显示
            //Text(String(Int(rotation)) + "度").font(.title)//通过UI直接查看animatableData的值
        }
        
        .rotation3DEffect(
            //rotation的值会一直更新，所以旋转的角度也会产生变化
            Angle.degrees(rotation),//使用rotation替换要旋转的值
            axis: (0,1,0)//分别为x轴、y轴、z轴
        )
    }
    //参数控制器也剪切过来
    private struct DrawingConstants{
        static let cornerRadius: CGFloat = 10//圆角值
        static let lineWidth: CGFloat = 3//strokeBorder的线宽
    }
}

extension View{
    func cardify(isFaceUp:Bool) -> some View {
        //在扩展里使用modifier替换掉在被修改的视图上的
        //这里的self还不知道去掉和保留的区别，目前效果都一样
        self.modifier(Cardify(isFaceUp: isFaceUp))
    }
}
