import SwiftUI//VM必须导入swiftUI包，使用的代码和数据类型，以CG开头的如CGFloat、CGSize等
//ViewModel 主要充当View与Model的连接作用
class EmojiMemoryGame:ObservableObject {
    typealias Card = MemoryGame<String>.Card
    //使用静态化成员本质上在此类里是全局性的，使用“类名.成员名”的方式直接访问,像结构体里嵌套了Card结构原理一样。
   private static let emojis = ["🪰","🦘","🐌","🐤","🦎","🐶","🐱","🐭","🐝","🏓","🥎","🏏","⛹🏼‍♀️","🚗","🦯","✈️","🚅","🚆","🚊","🚜","🛳","🚈","📀","🧭","🖨","⌚️","📡"]

    //使用函数创建并初始化游戏
    private static func createMemoryGame() -> MemoryGame<String>{
        //通过MemoryGame的初始化生成卡片的个数，然后通过闭包函数获取到内容表情(这里的pairIndex是函数形参)
        MemoryGame<String>(numberOfPairsOfCards: 21){ pairIndex in
            EmojiMemoryGame.emojis[pairIndex] //通过传入的实参值返回表情[序列]内容
        }
    }
    //创建MemoryGame的时候我们必须要像数组一样指定泛型的类型
    @Published private(set) var model:MemoryGame<String> = createMemoryGame()//创建与Model的数据链接,并保护起来免受View对数据的破坏
    //通过model保护数据，并通过cards让数据只读
    var cards:Array<Card>{
        model.cards
    }
    // MARK: - Intent(s) 用户意图
    func choose(_ card: Card) {
        withAnimation{
            model.choose(card)
        }
    }
    func shuffle(){
        model.shuffle()//调用Model里的洗牌功能
    }
func restart(){
    model = EmojiMemoryGame.createMemoryGame()//重新初始化游戏
}
}

